// -- FILE ------------------------------------------------------------------
// name       : PluginFactoryImpl.java
// project    : BoarderZone: Libraries
// created    : Leon Poyyayil - 2005.06.25
// language   : java
// environment: JDK 1.5.0
// copyright  : (c) 1990-2008 by Leon Poyyayil (private), Switzerland
// license    : this is free software licensed under the GPL. see COPYING
// --------------------------------------------------------------------------
package net.boarderzone.plugin;

import net.boarderzone.util.Dependencies;
import net.boarderzone.util.Dependency;
import net.boarderzone.util.DependencyXmlHelper;
import net.boarderzone.util.ExceptionTool;
import net.boarderzone.util.JavaTool;
import net.boarderzone.util.Logger;
import net.boarderzone.util.Version;
import net.boarderzone.util.VersionInfoBase;
import net.boarderzone.util.VersionMismatchException;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;

import org.xml.sax.SAXException;


// -- CLASS -----------------------------------------------------------------
/** A default implementation of the PluginFactory interface.
@author   Leon Poyyayil
@version  0.8.458
@since    0.4.225
*///-------------------------------------------------------------------------
public class PluginFactoryImpl implements PluginFactory {


// ----- constants -----
/** provides version information for this class/package */
public static final VersionInfo VERSION_INFO = VersionInfo.getInstance();


// -- METHOD ----------------------------------------------------------------
/** Creates a new instance with the given loader.
@since    0.4.225
@param    pRequirements the requirements a plugin must fulfill in order to be
          acceptable. may not be null.
@param    pLoader the loader to use to locate plugin classes. may not be null.
*///-------------------------------------------------------------------------
public PluginFactoryImpl(
  final PluginHostRequirements pRequirements,
  final PluginLoader           pLoader
)
{
  if ( pRequirements == null || pLoader == null ) {
    throw new NullPointerException();
  }
  mLoadedPluginClasses = new HashMap<String, Class>();
  mRequirements = pRequirements;
  mLoader = pLoader;
  LOGGER.debug( "instantiated with requirements: " + mRequirements );
} // PluginFactoryImpl


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    0.4.225
*///-------------------------------------------------------------------------
public Class loadPluginClass(
  final String pPluginClassName
) throws PluginIncompatibleException, PluginNotFoundException
{
  if ( pPluginClassName == null ) {
    throw new NullPointerException();
  }

  Class lPluginClass = mLoadedPluginClasses.get( pPluginClassName );
  if ( lPluginClass == null ) {
    LOGGER.debug( "loading plugin class " + pPluginClassName );
    lPluginClass = mLoader.loadPluginClass( pPluginClassName );

    // ----- verify host requirements -----
    // 1.) check the required interface
    if ( !JavaTool.isAssignableFrom( mRequirements.getPluginInterface(), lPluginClass ) ) {
      LOGGER.warn( "plugin " + pPluginClassName + " doesn't implement interface " +
        mRequirements.getPluginInterface().getName() );
      throw new PluginIncompatibleException( pPluginClassName,
        PluginIncompatibleException.DOESNT_IMPLEMENT_INTERFACE,
        mRequirements.getPluginInterface().getName() );
    }

    // 2.) check availability of a public no-arg c'tor
    if ( lPluginClass.isInterface() ) {
      LOGGER.warn( "plugin " + pPluginClassName + " is an interface" );
      throw new PluginIncompatibleException( pPluginClassName,
        PluginIncompatibleException.IS_INTERFACE );
    } else if ( Modifier.isAbstract( lPluginClass.getModifiers() ) ) {
      LOGGER.warn( "plugin " + pPluginClassName + " is an abstract class" );
      throw new PluginIncompatibleException( pPluginClassName,
        PluginIncompatibleException.IS_ABSTRACT );
    }
    try {
      lPluginClass.getConstructor( new Class[ 0 ] );
    } catch ( NoSuchMethodException pExc ) {
      LOGGER.warn( "plugin " + pPluginClassName + " has no public no-arg c'tor", pExc );
      throw new PluginIncompatibleException( pPluginClassName,
        PluginIncompatibleException.NO_CONSTRUCTOR );
    }

    // 3.) get VERSION_INFO field and match it against the required version
    try {
      final Field lVersionInfoField = lPluginClass.getDeclaredField( "VERSION_INFO" );
      try {
        final VersionInfoBase lVersionInfo = ( VersionInfoBase )lVersionInfoField.get( null );
        final Version lPluginVersion = lVersionInfo.getVersionFull();
        final Version lMinRequiredVersion = mRequirements.getMinRequiredVersion();
        if ( lPluginVersion.compareTo( lMinRequiredVersion ) < 0 ) {
          LOGGER.warn( "plugin " + pPluginClassName + " has invalid version: " +
            lPluginVersion.getVersion() + " but require " + lMinRequiredVersion.getVersion() );
          throw new PluginIncompatibleException( pPluginClassName,
            PluginIncompatibleException.TOO_OLD_PLUGIN_VERSION,
            new Object[]{ lPluginVersion.getVersion(), lMinRequiredVersion.getVersion() } );
        }
      } catch ( IllegalAccessException pExc ) {
        LOGGER.warn( "plugin " + pPluginClassName + " has inaccessible VERSION_INFO field", pExc );
        throw new PluginIncompatibleException( pPluginClassName,
          PluginIncompatibleException.INACCESSIBLE_VERSION_INFO_FIELD );
      } catch ( ClassCastException pExc ) {
        LOGGER.warn( "plugin " + pPluginClassName + " has invalid VERSION_INFO field", pExc );
        throw new PluginIncompatibleException( pPluginClassName,
          PluginIncompatibleException.INVALID_VERSION_INFO_FIELD,
          lVersionInfoField.getType().getName() );
      }
    } catch ( NoSuchFieldException pExc ) {
      LOGGER.warn( "plugin " + pPluginClassName + " has no VERSION_INFO field", pExc );
      throw new PluginIncompatibleException( pPluginClassName,
        PluginIncompatibleException.NO_VERSION_INFO_FIELD );
    }

    // ----- verify plugin requirements -----
    // 1.) check package dependencies
    final String lDependencyResName = JavaTool.getPureClassName( lPluginClass ) + "Dependency.xml";
    final InputStream lDependencyRes = lPluginClass.getResourceAsStream( lDependencyResName );
    if ( lDependencyRes != null ) {
      try {
        final Dependencies lDependencies = DependencyXmlHelper.parse( lDependencyRes );
        for ( final Dependency lDependency : lDependencies.iterable() ) {
          try {
            lDependency.check( VERSION_INFO, lPluginClass.getClassLoader() );
          } catch ( VersionMismatchException pExc ) {
            LOGGER.warn( "plugin " + pPluginClassName + " has dependency problems: " );
            LOGGER.warn( ExceptionTool.getMessageOf( pExc ) );
            throw new PluginIncompatibleException( pPluginClassName,
              PluginIncompatibleException.VERSION_MISMATCH,
              ExceptionTool.getMessageOf( pExc ), pExc );
          }
        }
      } catch ( IOException pExc ) {
        LOGGER.warn( "plugin " + pPluginClassName + " has error loading dependencies", pExc );
        throw new PluginIncompatibleException( pPluginClassName,
          PluginIncompatibleException.INACCESSIBLE_DEPENDENCIES,
          ExceptionTool.getMessageOf( pExc ), pExc );
      } catch ( SAXException pExc ) {
        LOGGER.warn( "plugin " + pPluginClassName + " has error loading dependencies", pExc );
        throw new PluginIncompatibleException( pPluginClassName,
          PluginIncompatibleException.INVALID_DEPENDENCIES,
          ExceptionTool.getMessageOf( pExc ), pExc );
      }
    } else {
      LOGGER.warn( "plugin " + pPluginClassName + " has no " +
        pPluginClassName + "Dependency.xml resource" );
    }

    mLoadedPluginClasses.put( pPluginClassName, lPluginClass );
    LOGGER.debug( "successfully loaded and verified plugin " + pPluginClassName );
  } else if ( LOGGER.isDebugEnabled() ) {
    LOGGER.debug( "plugin " + pPluginClassName + " already loaded" );
  }
  return lPluginClass;
} // loadPluginClass


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    0.4.225
*///-------------------------------------------------------------------------
public Object createPlugin(
  final String pPluginClassName
) throws PluginIncompatibleException, PluginNotFoundException
{
  final Class lPluginClass = loadPluginClass( pPluginClassName );
  try {
    LOGGER.debug( "instantiating plugin " + pPluginClassName );
    return lPluginClass.newInstance();
  } catch ( Exception pExc ) {
    LOGGER.warn( "plugin " + pPluginClassName + " could not be instantiated: " + pExc, pExc );
    throw new PluginIncompatibleException( pPluginClassName,
      PluginIncompatibleException.CANNOT_BE_INSTANTIATED,
      ExceptionTool.getMessageOf( pExc ), pExc );
  }
} // createPlugin


// ----- data members -----
/** the logger to use for this class */
private static final Logger LOGGER = Logger.getLogger( PluginFactoryImpl.class );

/** the cache of successfully loaded and verified plugin classes */
private HashMap<String, Class> mLoadedPluginClasses;
/** the requirements we have for a plugin */
private PluginHostRequirements mRequirements;
/** the loader to locate the plugin classes */
private PluginLoader mLoader;


} // class PluginFactoryImpl
// --------------------------------------------------------------------------


// -- EOF -------------------------------------------------------------------
