// -- FILE ------------------------------------------------------------------
// name       : PluginIncompatibleException.java
// project    : BoarderZone: Libraries
// created    : Leon Poyyayil - 2005.06.25
// language   : java
// environment: JDK 1.5.0
// copyright  : (c) 1990-2008 by Leon Poyyayil (private), Switzerland
// license    : this is free software licensed under the GPL. see COPYING
// --------------------------------------------------------------------------
package net.boarderzone.plugin;

import net.boarderzone.util.lang.Message;


// -- CLASS -----------------------------------------------------------------
/** Thrown by PluginFactory implementations when creation of a plugin fails.
@author   Leon Poyyayil
@version  0.8.458
@since    0.4.225
*///-------------------------------------------------------------------------
public final class PluginIncompatibleException extends PluginException {


// ----- constants -----
/** provides version information for this class/package */
public static final VersionInfo VERSION_INFO = VersionInfo.getInstance();


/** one of the possible message keys */
public static final String CANNOT_BE_INSTANTIATED          = "cannot_be_instantiated";
/** one of the possible message keys */
public static final String DOESNT_IMPLEMENT_INTERFACE      = "doesnt_implement_interface";
/** one of the possible message keys */
public static final String NO_VERSION_INFO_FIELD           = "no_version_info_field";
/** one of the possible message keys */
public static final String INACCESSIBLE_VERSION_INFO_FIELD = "inaccessible_version_info_field";
/** one of the possible message keys */
public static final String INVALID_VERSION_INFO_FIELD      = "invalid_version_info_field";
/** one of the possible message keys */
public static final String TOO_OLD_PLUGIN_VERSION          = "too_old_plugin_version";
/** one of the possible message keys */
public static final String IS_INTERFACE                    = "is_interface";
/** one of the possible message keys */
public static final String IS_ABSTRACT                     = "is_abstract";
/** one of the possible message keys */
public static final String NO_CONSTRUCTOR                  = "no_constructor";
/** one of the possible message keys */
public static final String INACCESSIBLE_DEPENDENCIES       = "inaccessible_dependencies";
/** one of the possible message keys */
public static final String INVALID_DEPENDENCIES            = "invalid_dependencies";
/** one of the possible message keys */
public static final String VERSION_MISMATCH                = "version_mismatch";


// -- METHOD ----------------------------------------------------------------
/** Explicit message constructor.
@since    0.4.225
@param    pPluginClassName the name of the plugin which is incompatible
@param    pReasonMessageKey the resource key for the reason message
*///-------------------------------------------------------------------------
public PluginIncompatibleException(
  final String pPluginClassName,
  final String pReasonMessageKey
)
{
  super( new Object[]{ pPluginClassName, loadMessage( pReasonMessageKey, null ) } );
} // PluginIncompatibleException


// -- METHOD ----------------------------------------------------------------
/** Explicit wrapper message constructor.
@since    0.4.225
@param    pPluginClassName the name of the plugin which is incompatible
@param    pReasonMessageKey the resource key for the reason message
@param    pCause the original exception which causes this exception
*///-------------------------------------------------------------------------
public PluginIncompatibleException(
  final String    pPluginClassName,
  final String    pReasonMessageKey,
  final Throwable pCause
)
{
  super( new Object[]{ pPluginClassName, loadMessage( pReasonMessageKey, null ) }, pCause );
} // PluginIncompatibleException


// -- METHOD ----------------------------------------------------------------
/** Explicit message constructor.
@since    0.4.225
@param    pPluginClassName the name of the plugin which is incompatible
@param    pReasonMessageKey the resource key for the reason message
@param    pMessageArgument the argument for the message
*///-------------------------------------------------------------------------
public PluginIncompatibleException(
  final String pPluginClassName,
  final String pReasonMessageKey,
  final String pMessageArgument
)
{
  this( pPluginClassName, pReasonMessageKey, new Object[]{ pMessageArgument } );
} // PluginIncompatibleException


// -- METHOD ----------------------------------------------------------------
/** Explicit message constructor.
@since    0.4.225
@param    pPluginClassName the name of the plugin which is incompatible
@param    pReasonMessageKey the resource key for the reason message
@param    pMessageArguments the arguments for the message
*///-------------------------------------------------------------------------
public PluginIncompatibleException(
  final String   pPluginClassName,
  final String   pReasonMessageKey,
  final Object[] pMessageArguments
)
{
  super( new Object[]{ pPluginClassName, loadMessage( pReasonMessageKey, pMessageArguments ) } );
} // PluginIncompatibleException


// -- METHOD ----------------------------------------------------------------
/** Explicit message constructor.
@since    0.4.225
@param    pPluginClassName the name of the plugin which is incompatible
@param    pReasonMessageKey the resource key for the reason message
@param    pMessageArgument the argument for the message
@param    pCause the original exception which causes this exception
*///-------------------------------------------------------------------------
public PluginIncompatibleException(
  final String    pPluginClassName,
  final String    pReasonMessageKey,
  final String    pMessageArgument,
  final Throwable pCause
)
{
  this( pPluginClassName, pReasonMessageKey, new Object[]{ pMessageArgument }, pCause );
} // PluginIncompatibleException


// -- METHOD ----------------------------------------------------------------
/** Explicit message constructor.
@since    0.4.225
@param    pPluginClassName the name of the plugin which is incompatible
@param    pReasonMessageKey the resource key for the reason message
@param    pMessageArguments the arguments for the message
@param    pCause the original exception which causes this exception
*///-------------------------------------------------------------------------
public PluginIncompatibleException(
  final String    pPluginClassName,
  final String    pReasonMessageKey,
  final Object[]  pMessageArguments,
  final Throwable pCause
)
{
  super( new Object[]{ pPluginClassName, loadMessage( pReasonMessageKey, pMessageArguments ) },
    pCause );
} // PluginIncompatibleException


// -- METHOD ----------------------------------------------------------------
/** Loads a message resource string and formats it with the given arguments.
@since    0.4.225
@param    pMessageKey the key of the message to load
@param    pMessageArgs the arguments to format with (may be null)
@return   the formatted message
*///-------------------------------------------------------------------------
private static String loadMessage(
  final String   pMessageKey,
  final Object[] pMessageArgs
)
{
  final Message lMessage = new Message( PluginIncompatibleException.class, pMessageKey );
  return pMessageArgs != null ? lMessage.format( pMessageArgs ) : lMessage.format();
} // loadMessage


// ----- members -----
/** serialization versioning: necessary to prevent obfuscation from breaking it */
private static final long serialVersionUID = 0xe88ab2a821a9684cL;


} // class PluginIncompatibleException
// --------------------------------------------------------------------------


// -- EOF -------------------------------------------------------------------
